/* gincoming.c   */
/* $Date: 1999/05/28 21:11:33 $ */
/* $Id: gincoming.c,v 0.12 1999/05/28 21:11:33 webweazl Exp webweazl $ */

/* -- Written by Frank Wiles <frank@wiles.org>  */
/* -- Bastardized by Nick Studt <nick@angrydwarf.org> */ 
/* -- and by D. Hageman <dhageman@dracken.com> */ 

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

/* those lovely Defines */
#define GINCOMING_VERSION "$Revision: 0.12 $"
#define GINCOMING_VERSION_DATE "$Date: 1999/05/28 21:11:33 $"
#define GINCOMING_URL "http://www.angrydwarf.org/GIncoming/"
#define GINCOMING_COPYRIGHT "Copyright (C) 1999 Frank, Dave and Nick"

/* Function Prototypes */
void add_to_list( char *name );
void alert_dialog( void );
gint delete_event( GtkWidget *widget, GdkEvent *event, gpointer data );
void destroy ( GtkWidget *widget, gpointer data );
void directory_dialog( void );
void directory_change( GtkWidget *widget, gpointer data);
void directory_warning(char *dir);
void display_about();
void display_help();
void display_warning();
void init_numfiles( void );
void get_main_menu(GtkWidget *window, GtkWidget **menubar );
int grab_files( void );
void print_version( void );
void print_help( void );
void update( void );
void update_dialog_update_value( GtkWidget *widget, gpointer data);
void update_dialog_window( void );
void update_func( GtkWidget *window, gpointer data );
gint watch();


/* Global GTK Vars */
GtkWidget *window;                  /* Main Window     */
GtkWidget *top_frame;               /* Frame at top    */
GtkWidget *clist;                   /* Our clist       */
GtkWidget *scrolled_window;         /* Scrolled Window */
GtkWidget *vbox;                    /* Vertical Box    */
GtkWidget *menu_bar;
GtkWidget *border_box;
GtkWidget *update_dialog;
GtkWidget *update_textentry;
GtkWidget *directory_widget;
GtkWidget *directory_textentry;

/* Global Vars */
char directory[750] = "/home/ftp/incoming/";
int numfiles = 0;
int timeout = 5000;
gint timeout_tag = 0;

/* the menus definition */
static GtkItemFactoryEntry menu_items[] = {
    { "/File",        "<control>F", NULL,          0, "<Branch>"             },
    { "/File/Update", NULL,         update_func,   0, NULL                   },
    { "/File/sep1",   NULL,         NULL,          0, "<Separator>"          },
    { "/File/Quit",   "<control>Q", gtk_main_quit, 0, NULL                   },
    { "/Options",     "<control>O", NULL,          0, "<Branch>"             },
    { "/Options/Change Directory", "<control>D", directory_dialog,   0, NULL },
    { "/Options/Change Refresh", "<control>R", update_dialog_window, 0, NULL },
    { "/Help",        NULL,         NULL,          0, "<LastBranch>"         },
    { "/Help/About...",   NULL,     display_about, 0, NULL                   },
    { "/Help/Help",   NULL,         display_help,  0, NULL                   },
};

/* Main */
int main(int argc, char *argv[]) {
    char titles[750] = {'\0'};
    struct stat buf;
    int loop;
    extern char *optarg;
    extern int optind;
    int ch; 

    gtk_init(&argc, &argv);

    while(( ch = getopt(argc, argv, "hvd:")) != -1) 
        switch(ch) {
        case 'v':
            print_version();
            exit(0);
            break;
        case 'd':
		  strcpy(directory , optarg);
		  if(directory[strlen(directory) - 1] != '/' ) {
			strcat(directory, "/");
		  }
		  break;
        case '?':
        case 'h':
        default:
            print_version();
            print_help();
            exit(0);
        }
 
    argc -= optind;
    argv += optind; 

    /* Check to make sure we can open this directory */
    if (opendir(directory) == NULL)   {
        fprintf (stderr, "%s: %s %s.\n", gettext("Error"), directory,
                 gettext("Can not be opened") );
        print_help();        
        exit(1);
    }

    /* Start our windows and stuff */
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_default_size( GTK_WINDOW (window) , 470 , 150 );

    /* Setup out window */
    gtk_window_set_title( GTK_WINDOW (window), "GIncoming" );
    gtk_signal_connect( GTK_OBJECT (window), "delete_event", 
                        GTK_SIGNAL_FUNC (delete_event), NULL);
    gtk_signal_connect( GTK_OBJECT (window), "destroy",
                        GTK_SIGNAL_FUNC (destroy), NULL);

    /* Vbox */
    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add( GTK_CONTAINER (window), vbox );
    gtk_widget_show(vbox);

    /* Menu */
    get_main_menu(window, &menu_bar);
    gtk_box_pack_start( GTK_BOX (vbox), menu_bar, FALSE, FALSE, 0);
    gtk_widget_show(menu_bar);

    /* Border box */
    border_box = gtk_vbox_new(FALSE,0);
    gtk_box_pack_start( GTK_BOX (vbox), border_box, TRUE, TRUE, 0);
    gtk_container_set_border_width( GTK_CONTAINER (border_box), 10);
    gtk_widget_show(border_box);


    /* Configure Top Frame */
    strcat(titles, "Directory ");
    strcat(titles, directory);
    top_frame = gtk_frame_new(titles);
    gtk_container_add( GTK_CONTAINER (border_box), top_frame);
    gtk_widget_show(top_frame);

    /* Scrolled Window */
    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW ( scrolled_window ),  
                                    GTK_POLICY_AUTOMATIC, 
									GTK_POLICY_AUTOMATIC );
    gtk_container_set_border_width( GTK_CONTAINER(scrolled_window), 10 );
    gtk_container_add( GTK_CONTAINER (top_frame), scrolled_window );
    gtk_widget_show(scrolled_window);

    /* Clist */
    clist = gtk_clist_new(3);
    gtk_clist_column_titles_passive(GTK_CLIST (clist));
    gtk_clist_set_column_title(GTK_CLIST (clist), 0, "Name");
    gtk_clist_set_column_title(GTK_CLIST (clist), 1, "Size");
    gtk_clist_set_column_title(GTK_CLIST (clist), 2, "Date");

    gtk_clist_set_column_justification( GTK_CLIST (clist), 1, 
                                        GTK_JUSTIFY_RIGHT );

    gtk_clist_column_titles_show(GTK_CLIST (clist));

    gtk_clist_set_column_width( GTK_CLIST (clist), 0, 140);
    gtk_clist_set_column_width( GTK_CLIST (clist), 1, 60);
    gtk_clist_set_column_width( GTK_CLIST (clist), 2, 60);

    gtk_container_add( GTK_CONTAINER (scrolled_window), clist );
    gtk_widget_show(clist);

    /* Pack */
    gtk_widget_show(window);
    timeout_tag = gtk_timeout_add(timeout, watch, NULL);

    /* Set out orginal value of num files */
    init_numfiles();

    /* Run update to populate our list for the first time */   
    update();

    /* Loop */
    gtk_main();

    /* end of Main */
    return(0);
} 

/* ################### watch ################### */
gint watch(  ) {
  if(numfiles < grab_files() ) {
	update();
	numfiles = grab_files();
	alert_dialog();
  }

  if(numfiles > grab_files() ) {
	update();
	numfiles = grab_files();
  }
  
  return (TRUE);
}


/* ###############  update_dialog_update_value  ############### */
void update_dialog_update_value( GtkWidget *widget, gpointer data) {

    int new_timeout;
    gchar *string;

    string = gtk_entry_get_text( GTK_ENTRY (update_textentry) );
    new_timeout = atoi(string);     

    /* Checks to make sure they aren't checking less than
       once a second */
    if( new_timeout < 1000 ) {

        display_warning();

    }
    else {
        timeout = new_timeout;
        gtk_timeout_remove(timeout_tag);
        timeout_tag = gtk_timeout_add(timeout, watch, NULL);
    }

    /* re-run our update just to be sure */
    update();
    gtk_widget_destroy(update_dialog);  
}

/* ###################  update_dialog  ################### */
void update_dialog_window() { 

    GtkWidget *label;
    GtkWidget *okbutton;
    GtkWidget *cancelbutton;
    char update_string[50] = {'\0'};

    update_dialog = gtk_dialog_new();
    okbutton = gtk_button_new_with_label("Ok");
    cancelbutton = gtk_button_new_with_label("Cancel");
    label = gtk_label_new("Enter new update time:");

    update_textentry = gtk_entry_new_with_max_length(47);
    sprintf( update_string, "%d", timeout );
    gtk_entry_set_text( GTK_ENTRY (update_textentry), update_string);
    
    gtk_container_set_border_width( GTK_CONTAINER 
									(GTK_DIALOG (update_dialog)->vbox), 10);

    gtk_box_pack_start( GTK_BOX 
						(GTK_DIALOG (update_dialog)->action_area), okbutton,
                        TRUE, TRUE, 0);

    gtk_box_pack_start( GTK_BOX (GTK_DIALOG (update_dialog)->action_area),
                        cancelbutton, TRUE, TRUE, 0);

    gtk_box_pack_start( GTK_BOX (GTK_DIALOG (update_dialog)->vbox), label,
                        TRUE, TRUE, 0);

    gtk_box_pack_start( GTK_BOX (GTK_DIALOG 
						(update_dialog)->vbox), update_textentry,
                        TRUE, TRUE, 0);

    gtk_signal_connect_object(  GTK_OBJECT (cancelbutton) , "clicked", 
                                GTK_SIGNAL_FUNC(gtk_widget_destroy),
                                GTK_OBJECT (update_dialog) );

    gtk_signal_connect(  GTK_OBJECT (okbutton), "clicked",
                         GTK_SIGNAL_FUNC(update_dialog_update_value),
                         NULL );
     
    GTK_WIDGET_SET_FLAGS( GTK_OBJECT(okbutton), GTK_CAN_DEFAULT );
    gtk_widget_grab_default(okbutton);
       
    gtk_widget_show(update_dialog);
    gtk_widget_show(update_textentry);
    gtk_widget_show(label);
    gtk_widget_show(okbutton);
    gtk_widget_show(cancelbutton);  

}
        
/* ###################  directory_dialog  ################### */
void directory_dialog() { 

    GtkWidget *label;
    GtkWidget *okbutton;
    GtkWidget *cancelbutton;

    directory_widget = gtk_dialog_new();
    okbutton = gtk_button_new_with_label("Ok");
    cancelbutton = gtk_button_new_with_label("Cancel");
    label = gtk_label_new("Enter Directory to view:");
    directory_textentry = gtk_entry_new();
    gtk_entry_set_text( GTK_ENTRY (directory_textentry), directory );
    gtk_entry_select_region( GTK_ENTRY (directory_textentry), 0,
                              strlen(directory) );

    gtk_container_set_border_width(GTK_CONTAINER 
								   (GTK_DIALOG (directory_widget)->vbox), 10);

    gtk_box_pack_start(GTK_BOX 
					   (GTK_DIALOG (directory_widget)->action_area), okbutton,
					   TRUE, TRUE, 0);

    gtk_box_pack_start( GTK_BOX (GTK_DIALOG (directory_widget)->action_area),
                        cancelbutton, TRUE, TRUE, 0);

    gtk_box_pack_start( GTK_BOX (GTK_DIALOG (directory_widget)->vbox), label,
                        TRUE, TRUE, 0);
    gtk_box_pack_start( GTK_BOX (GTK_DIALOG (directory_widget)->vbox), 
						directory_textentry, TRUE, TRUE, 0);

    gtk_signal_connect_object(  GTK_OBJECT (cancelbutton) , "clicked", 
                                GTK_SIGNAL_FUNC(gtk_widget_destroy),
                                GTK_OBJECT (directory_widget));

    gtk_signal_connect(  GTK_OBJECT (okbutton), "clicked",
                         GTK_SIGNAL_FUNC(directory_change),
                         NULL );

    GTK_WIDGET_SET_FLAGS( GTK_OBJECT (okbutton), GTK_CAN_DEFAULT );
    gtk_widget_grab_default(okbutton);

    gtk_widget_show(directory_widget);
    gtk_widget_show(directory_textentry);
    gtk_widget_show(label);
    gtk_widget_show(okbutton);
    gtk_widget_show(cancelbutton);  
        
}

/* ###################  directory_change  ################### */
void directory_change( GtkWidget *widget, gpointer data) {

    char *temp;
    char frame_label[600];

    temp = gtk_entry_get_text( GTK_ENTRY (directory_textentry) );

    if( temp[ strlen(temp) - 1 ] != '/' ) {
        strcat(temp, "/");
    }
    
    if (opendir(temp) == NULL)   {
        fprintf (stderr, "%s: %s %s.\n", gettext("Error"), temp,
                 gettext("Can not be opened") );
    
        directory_warning(temp);    
    } else {

        strcpy(directory ,temp);
        numfiles = grab_files();
        
        sprintf(frame_label, "Directory: %s", directory);        
        gtk_frame_set_label( GTK_FRAME (top_frame), frame_label );
    }
 
    gtk_widget_destroy(directory_widget);
    update();   
}

/* ###################  alert_dialog  ################### */
void alert_dialog( void ) {
    char dialog_text[1000] = {'\0'};
    GtkWidget *dialog; 
    GtkWidget *alert_label;
    GtkWidget *button;

    dialog = gtk_dialog_new();

    strcat(dialog_text,"There is something new for you\nin ");
	strcat(dialog_text,directory);
    strcat(dialog_text,"..." );
    
    alert_label = gtk_label_new(dialog_text);
    button = gtk_button_new_with_label("Ok");

    gtk_container_set_border_width(GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 
                                   15);

    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (dialog)->vbox), alert_label,
                       TRUE, TRUE, 0);

    gtk_box_pack_start( GTK_BOX ( GTK_DIALOG (dialog)->action_area ), button,
                        TRUE, FALSE, 0);

    gtk_signal_connect_object(  GTK_OBJECT (button) , "clicked", 
                                GTK_SIGNAL_FUNC(gtk_widget_destroy),
                                GTK_OBJECT (dialog) );
    gtk_widget_show(button);
    gtk_widget_show(alert_label);       
    gtk_widget_show(dialog); 
}

/* *******************  update_button  ******************* */
void update_func( GtkWidget *window, gpointer data ) {
    watch();
}


/* *******************  display_about  ******************* */
void display_about() {

    GtkWidget *dialog;
    GtkWidget *frame; 
    GtkWidget *label;
    GtkWidget *button;
 
    dialog = gtk_dialog_new();
    frame = gtk_frame_new("About GIncoming...");
    gtk_container_set_border_width( GTK_CONTAINER (frame), 5);
    label = gtk_label_new("GIncoming v 0.12\n\nCopyright (c) 1999 by Frank Wiles,\n Dave Hageman, and Nick Studt.");

    gtk_container_set_border_width( GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
                                    15);

    gtk_container_add( GTK_CONTAINER (frame), label );  
    gtk_box_pack_start( GTK_BOX (GTK_DIALOG (dialog)->vbox), frame,
                        TRUE, TRUE, 0);
   
    button = gtk_button_new_with_label("Ok");
    gtk_box_pack_start( GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
                        FALSE, FALSE, 0);

    gtk_signal_connect_object( GTK_OBJECT(button), "clicked",
                               GTK_SIGNAL_FUNC (gtk_widget_destroy),
                               GTK_OBJECT (dialog) );                           

    GTK_WIDGET_SET_FLAGS( GTK_OBJECT(button), GTK_CAN_DEFAULT );
    gtk_widget_grab_default(button);

    gtk_widget_show(button);
    gtk_widget_show(label);
    gtk_widget_show(frame);
    gtk_widget_show(dialog);

    
}

/* *******************  display_help  ******************* */
void display_help() {

    GtkWidget *window;
    GtkWidget *frame;
    GtkWidget *vbox;
    GtkWidget *label; 
 
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_default_size( GTK_WINDOW (window) , 400 , 150 );
    gtk_window_set_title( GTK_WINDOW (window), "GIncoming -- Help" );
    gtk_container_set_border_width( GTK_CONTAINER (window), 10 );

    vbox = gtk_vbox_new(FALSE, 0);
    frame = gtk_frame_new("GIncoming Help");
    gtk_container_set_border_width( GTK_CONTAINER (frame), 5 );

    label = gtk_label_new("\n   1) Use the File->Update menu item to force GIncoming to update the screen.\n   2) Use the Options->Change Directory menu\n       option to change the directory GIncoming monitors.\n   3) Use the Options->Change Refresh menu option\n       to change how often GIncoming updates the screen in milliseconds.\n");

    gtk_label_set_justify( GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start( GTK_BOX (vbox), frame, FALSE, TRUE, 0 );

    gtk_container_add( GTK_CONTAINER (vbox), frame);
    gtk_container_add( GTK_CONTAINER (window), vbox);
    gtk_container_add( GTK_CONTAINER (frame), label);

    gtk_widget_show(label);
    gtk_widget_show(frame);
    gtk_widget_show(vbox);
    gtk_widget_show(window);

}

/* ###################  display_warning  ################### */
void display_warning() {

    GtkWidget *dialog;
    GtkWidget *label;
    GtkWidget *button;

    dialog = gtk_dialog_new();
    label = gtk_label_new("Sorry! Update value must be greater than 1000 milliseconds");
    button = gtk_button_new_with_label("Ok");

    gtk_container_set_border_width( GTK_CONTAINER ( GTK_DIALOG (dialog)->vbox ), 15 );

    gtk_box_pack_start( GTK_BOX (GTK_DIALOG (dialog)->vbox ), label,
                        TRUE, TRUE, 0 );
    gtk_box_pack_start( GTK_BOX (GTK_DIALOG (dialog)->action_area ) , button,
                        TRUE, FALSE, 0 );   

    gtk_signal_connect_object( GTK_OBJECT(button), "clicked",
                               GTK_SIGNAL_FUNC (gtk_widget_destroy),
                               GTK_OBJECT (dialog) );

    gtk_widget_show(button);
    gtk_widget_show(label);
    gtk_widget_show(dialog);
}

/* ###################  init_numfiles  ################### */
void init_numfiles( void ) {
    DIR *incoming;
    int counter = 0;

    incoming = opendir(directory);

    while( readdir(incoming) != NULL) {
           counter++;
    }

    numfiles = counter;
    closedir(incoming);
}

/* *******************  get_main_menu  ******************* */
void get_main_menu(GtkWidget *window, GtkWidget **menubar ) {

    GtkItemFactory *item_factory;
    GtkAccelGroup  *accel_group;
    gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);

    accel_group = gtk_accel_group_new();

    item_factory = gtk_item_factory_new( GTK_TYPE_MENU_BAR, "<main>",
                                         accel_group );
    gtk_item_factory_create_items( item_factory, nmenu_items, menu_items,
                                   NULL );
    gtk_accel_group_attach( accel_group, GTK_OBJECT (window) );

    if(menubar)
        *menubar = gtk_item_factory_get_widget( item_factory, "<main>" );
    
}


/* ################### grab_files ################### */
int grab_files( void ) {
    DIR *incoming;
    int counter = 0;

    incoming = opendir(directory);
    
    while( readdir(incoming) != NULL ) { 
           counter++; }

    closedir(incoming);

    return(counter);
}

/* ################### add_to_list ################### */
void add_to_list(char *name) {
    struct stat filestruct;
    char fqpath[500] = {'\0'};

    int status;

    gchar *addition[1][3];


    strcat(fqpath, directory);
    strcat(fqpath, name);

    status = stat(fqpath, &filestruct);

    if (status == 0)  {
        /*  Create a temp var, that will hold the end formated size string */
        char size[15] = {'\0'};

        /*  Type cast to the type of variable we need */
        double tempdoublesize = (unsigned long int)filestruct.st_size;

        sprintf(size, "%0.f", tempdoublesize );

        addition[0][0] = name;
        addition[0][1] = size;
        addition[0][2] = ctime(&filestruct.st_mtime);

        if( strcmp(name, ".")  == 0 ) { return; }
        if( strcmp(name, "..") == 0 ) { return; }

        gtk_clist_append( GTK_CLIST (clist), addition[0] );
        return;
    }
}

/* ################### update ################### */
void update( void ) {
    DIR *incoming;
    struct dirent *file;

    incoming = opendir(directory);

    /* Clear out clist */
    gtk_clist_clear( GTK_CLIST (clist) );

    file = readdir(incoming);
    
    while( file != NULL ) {
        add_to_list(file->d_name);
        file = readdir(incoming);
    }
    closedir(incoming);
}


/* ################### delete_event *################### */
gint delete_event( GtkWidget *widget, GdkEvent *event, gpointer data ) {
    return(FALSE);
} 

/* ################### destroy ################### */
void destroy ( GtkWidget *widget, gpointer data ) {
    gtk_main_quit();
}

/* ################### print Version ################### */
void print_version( void ) {
    printf( "GIncoming %s (%s)\n", GINCOMING_VERSION, GINCOMING_VERSION_DATE );
    printf( "%s: %d.%d.%d\n", gettext ( "GTK+ runtime" ),
             gtk_major_version, gtk_minor_version, gtk_micro_version );
    printf( "%s: %s\n", gettext ( "Home page" ), GINCOMING_URL );
    printf( "%s\n", GINCOMING_COPYRIGHT );
}

/* ################### print Help ################### */
void print_help( void ) { 
    printf( "Gincoming %s:\n", gettext ( "options" ) );
    printf( "%-30s %s\n", "-h",
            gettext ( "display this help info" ) );
    printf( "%-30s %s\n", "-v",
            gettext ( "display the version" ) );
    printf( "%-30s %s\n", "-d directory",
            gettext ( "specify the directory to watch" ) );
}

/* ################### directory_warning ################### */
void directory_warning(char *dir) {

    GtkWidget *dialog;
    GtkWidget *frame;
    GtkWidget *label;
    GtkWidget *okbutton;
    char string[500] = {'\0'};

    dialog = gtk_dialog_new();
    frame = gtk_frame_new("Error:");

    strcat(string, "Sorry the directory ");
    strcat(string, dir);
    strcat(string, " could not be opened.");
    label = gtk_label_new(string);

    gtk_box_pack_start( GTK_BOX( GTK_DIALOG (dialog)->vbox), frame, 
                        TRUE, TRUE, 0);
    gtk_container_set_border_width( GTK_CONTAINER(frame), 5 );
    gtk_container_set_border_width( GTK_CONTAINER( GTK_DIALOG (dialog)->vbox), 
                                    10 );

    gtk_container_add( GTK_CONTAINER (frame), label);

    okbutton = gtk_button_new_with_label("Ok");

    gtk_box_pack_start( GTK_BOX( GTK_DIALOG (dialog)->action_area), okbutton,
                        TRUE, TRUE, 0 );

    gtk_signal_connect_object( GTK_OBJECT(okbutton), "clicked",
                               GTK_SIGNAL_FUNC (gtk_widget_destroy),
                               GTK_OBJECT (dialog) );

    gtk_widget_show(label);
    gtk_widget_show(okbutton);
    gtk_widget_show(frame);
    gtk_widget_show(dialog);

}

/* EOF */
